import { useState, useEffect, createElement } from 'react';

/**
 * A factory function for creating asynchronous React components
 *
 * @param componentGetter A function that returns a React component or a promise that resolves a React component
 * @param options Optionally adds `loading` and `error` state components
 */

var __reactCreateElement__ = createElement;

function createAsyncComponent(componentGetter, options) {
  if (options === void 0) {
    options = {};
  }

  var {
    loading,
    error
  } = options;
  var cachedComponent;
  var PROD = typeof process !== 'undefined' && "production" === 'production';

  function _ref() {
    return cachedComponent;
  }

  function _ref4(component) {
    return cachedComponent = component;
  }

  return Object.assign(function AsyncComponent(props) {
    var [Component, setComponent] = useState(cachedComponent ? _ref : componentGetter);
    var [exception, setException] = useState(null);

    function _ref2(component) {
      if (PROD) cachedComponent = component;
      setComponent(() => component);
    }

    function _ref3(err) {
      return setException(err);
    }

    useEffect(() => {
      if ('then' in Component) Component.then(_ref2).catch(_ref3); // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return exception ? (error === null || error === void 0 ? void 0 : error(exception, props)) || null : 'then' in Component ? (loading === null || loading === void 0 ? void 0 : loading(props)) || null : __reactCreateElement__(Component, props);
  }, {
    load: () => {
      if (PROD) return cachedComponent ? Promise.resolve(cachedComponent) : Promise.resolve(componentGetter()).then(_ref4);
      return Promise.resolve(componentGetter());
    }
  });
}

export default createAsyncComponent;
//# sourceMappingURL=index.dev.mjs.map
